@@ -77,7 +77,9 @@ class EmbedAuthor(TypedDict, total=False):
     proxy_icon_url: str
 
 
-EmbedType = Literal["rich", "image", "video", "gifv", "article", "link", "auto_moderation_message"]
+EmbedType = Literal[
+    "rich", "image", "video", "gifv", "article", "link", "auto_moderation_message"
+]
 
 
 class Embed(TypedDict, total=False):
