@@ -4,45 +4,42 @@
     _BaseGUILD class.
 """
 from __future__ import annotations
-import asyncio
+
+from typing import Any, Coroutine, Union, List, Optional, Dict, Callable
 from contextlib import suppress
-from typing import Any, Coroutine, Union, List, Optional, Dict, Callable, TypeVar
 from typeguard import typechecked
 from datetime import timedelta, datetime
+from enum import Enum
+from copy import deepcopy
 
-from .exceptions import *
-from .tracing import *
-from .common import *
+from .logging.tracing import *
 from .message import *
 
 from . import client
-from . import sql
-from . import core
+from .logging import sql
 from . import misc
+from .logging import logging
 
 import _discord as discord
-import time
-import json
-import pathlib
-import shutil
 import operator
+import asyncio
+import re
 
 
 __all__ = (
     "GUILD",
-    "USER"
+    "USER",
+    "AutoGUILD"
 )
 
-#######################################################################
-# Globals
-#######################################################################
-class GLOBALS:
+class AdvertiseTaskType(Enum):
     """
-    Contains the global variables for the module.
+    Used for identifying advertiser tasks
     """
-    server_log_path = None
+    TEXT_ISH = 0
+    VOICE = 1
+
 
-@typechecked
 class _BaseGUILD:
     """
     Represents an universal guild.
@@ -74,9 +71,9 @@ class _BaseGUILD:
         "_messages_uninitialized",
         "message_dict",
         "remove_after",
-        "_created_at"
+        "_created_at",
+        "_deleted"
     )
-    __logname__ = "_BaseGUILD" # Dummy to demonstrate correct definition for @sql._register_type decorator
     
     def __init__(self,
                  snowflake: Any,
@@ -89,20 +86,12 @@ class _BaseGUILD:
         self._messages_uninitialized: list = messages   # Contains all the different message objects, this gets sorted in `.initialize()` method
         self.message_dict: Dict[str, List[BaseMESSAGE]] = {AdvertiseTaskType.TEXT_ISH: [], AdvertiseTaskType.VOICE: []}  # Dictionary, which's keys hold the discord message type(text, voice) and keys are a list of messages
         self.remove_after = remove_after  # int - after n sends; timedelta - after amount of time; datetime - after that time
+        self._deleted = False
         self._created_at = datetime.now() # The time this object was created
 
     def __repr__(self) -> str:
         return f"{type(self).__name__}(discord={self.apiobject})"
 
-    @property
-    def _log_file_name(self):
-        """
-        Returns a string that transforms the xGUILD's discord name into
-        a string that contains only allowed character. This is a method instead of
-        property because the name can change overtime.
-        """
-        raise NotImplementedError
-
     @property
     def messages(self) -> List[BaseMESSAGE]:
         """
@@ -151,6 +140,12 @@ class _BaseGUILD:
         """
         return self.snowflake == other.snowflake
 
+    def _delete(self):
+        """
+        Sets the internal _deleted flag to True.
+        """
+        self._deleted = True
+
     async def add_message(self, message: BaseMESSAGE):
         """
         Adds a message to the message list.
@@ -179,7 +174,7 @@ class _BaseGUILD:
 
         Raises
         -----------
-        DAFNotFoundError(code=DAF_SNOWFLAKE_NOT_FOUND)
+        ValueError
             Raised when the guild_id wasn't found.
         Other
             Raised from .add_message(message_object) method.
@@ -194,14 +189,14 @@ class _BaseGUILD:
             for message in self._messages_uninitialized:
                 try:
                     await self.add_message(message)
-                except (TypeError, ValueError, DAFError) as exc:
+                except (TypeError, ValueError) as exc:
                     trace(f"[GUILD:] Unable to initialize message {message}, in {self}\nReason: {exc}", TraceLEVELS.WARNING)
 
 
             self._messages_uninitialized.clear()
             return
 
-        raise DAFNotFoundError(f"Unable to find object with ID: {guild_id}", DAF_SNOWFLAKE_NOT_FOUND)
+        raise ValueError(f"Unable to find object with ID: {guild_id}")
 
     def remove_message(self, message: BaseMESSAGE):
         """
@@ -245,18 +240,13 @@ class _BaseGUILD:
         mode: AdvertiseTaskType
             Tells which task called this method (there is one task for textual messages and one for voice like messages).
         """
-        # Check if the guild has expired and check if the guild is in the list (could not be due to asynchronous operations)
-        if self._check_state() and self in core.get_shill_list():
-            trace(f"[GUILD:] Removing {self}")
-            core.remove_object(self)
-            return
-
         msg_list = self.message_dict[mode]
         for message in msg_list[:]: # Copy the avoid issues with the list being modified while iterating (add_message/remove_message)
             # Message removal             Check due to asynchronous operations
-            if message._check_state() and message in self.messages:
-                trace(f"[GUILD:] Removing {message} which is part of {self}")
-                self.remove_message(message)
+            if message._check_state():
+                # Suppress since user could of called the remove_object function mid iteration.
+                with suppress(ValueError):
+                    self.remove_message(message)
 
             elif message._is_ready():
                 message._reset_timer()
@@ -264,92 +254,25 @@ class _BaseGUILD:
 
                 # Generate log (JSON or SQL)
                 if self.logging and message_ret is not None:
-                    await self._generate_log(message_ret)
+                    await logging.save_log(self.generate_log_context(), message_ret)
 
-    async def _generate_log(self,
-                           message_context: dict) -> None:
+    def generate_log_context(self) -> Dict[str, Union[str, int]]:
         """
-        Generates a .json type log or logs to a SQL database for each message send attempt.
+        Generates a dictionary of the guild's context,
+        which is then used for logging.
 
-        Prioritizes SQL logging.
-
-        Parameter:
-        -----------
-        data_context: dict
-            Dictionary containing data describing the message send attempt. (Return of ``message._send()``)
+        Returns
+        ---------
+        Dict[str, Union[str, int]]
         """
-        guild_context = {
-            "name" : str(self.apiobject),
-            "id" : self.snowflake,
-            "type" : type(self).__logname__,
+        return {
+            "name": self.apiobject.name,
+            "id": self.apiobject.id,
+            "type": type(self).__name__
         }
 
-        try:
-            # Try to obtain the sql manager. If it returns None (sql logging disabled), save into file
-            manager = sql.get_sql_manager()
-            if (
-                manager is None or  # Short circuit evaluation
-                not await manager._save_log(guild_context, message_context)
-            ):
-                timestruct = time.localtime()
-                timestamp = "{:02d}.{:02d}.{:04d} {:02d}:{:02d}:{:02d}".format(timestruct.tm_mday,
-                                                                            timestruct.tm_mon,
-                                                                            timestruct.tm_year,
-                                                                            timestruct.tm_hour,
-                                                                            timestruct.tm_min,
-                                                                            timestruct.tm_sec)
-                logging_output = pathlib.Path(GLOBALS.server_log_path)\
-                            .joinpath("{:02d}".format(timestruct.tm_year))\
-                            .joinpath("{:02d}".format(timestruct.tm_mon))\
-                            .joinpath("{:02d}".format(timestruct.tm_mday))
-                with suppress(FileExistsError):
-                    logging_output.mkdir(parents=True,exist_ok=True)
-
-                logging_output = str(logging_output.joinpath(self._log_file_name))
-
-                # Create file if it doesn't exist
-                fresh_file = False
-                with suppress(FileExistsError), open(logging_output, "x", encoding="utf-8"):
-                    fresh_file = True
-
-                # Write to file
-                with open(logging_output,'r+', encoding='utf-8') as appender:
-                    appender_data = None
-                    appender.seek(0) # Append moves cursor to the end of the file
-                    try:
-                        appender_data: dict = json.load(appender)
-                    except json.JSONDecodeError:
-                        # No valid json in the file, create new data
-                        # and create a .old file to store this invalid data
-                        # Copy-paste to .old file to prevent data loss
-                        if not fresh_file: ## Failed because the file was just created, no need to copy-paste
-                            # File not newly created, and has invalid data
-                            shutil.copyfile(logging_output, f"{logging_output}.old")
-                        # Create new data
-                        appender_data = {}
-                        appender_data["name"] = guild_context["name"]
-                        appender_data["id"] = guild_context["id"]
-                        appender_data["type"] = guild_context["type"]
-                        appender_data["message_history"] = []
-                    finally:
-                        appender.seek(0) # Reset cursor to the beginning of the file after reading
-
-                    appender_data["message_history"].insert(0,
-                        {
-                            **message_context,
-                            "index": appender_data["message_history"][0]["index"] + 1 if len(appender_data["message_history"]) else 0,
-                            "timestamp": timestamp
-                        })
-                    json.dump(appender_data, appender, indent=4)
-                    appender.truncate() # Remove any old data
-
-        except Exception as exception:
-            # Any uncaught exception (prevent from complete framework stop)
-            trace(f"[{type(self).__name__}]: Unable to save log. Exception: {exception}", TraceLEVELS.WARNING)
-
-
-@typechecked
-@sql._register_type("GuildTYPE")
+@misc.doc_category("Guilds")
+@sql.register_type("GuildTYPE")
 class GUILD(_BaseGUILD):
     """
     The GUILD object represents a server to which messages will be sent.
@@ -373,12 +296,11 @@ class GUILD(_BaseGUILD):
         * timedelta - the specified time difference
         * datetime - specific date & time
     """
-
-    __logname__ = "GUILD" # For sql._register_type
     __slots__ = (
         "update_semaphore",
     )
 
+    @typechecked
     def __init__(self,
                  snowflake: Union[int, discord.Guild],
                  messages: Optional[List[Union[TextMESSAGE, VoiceMESSAGE]]]=[],
@@ -387,10 +309,7 @@ class GUILD(_BaseGUILD):
         super().__init__(snowflake, messages, logging, remove_after)
         misc._write_attr_once(self, "update_semaphore", asyncio.Semaphore(2))
 
-    @property
-    def _log_file_name(self):
-        return "".join(char if char not in C_FILE_NAME_FORBIDDEN_CHAR else "#" for char in self.apiobject.name) + ".json"
-
+    @typechecked
     async def add_message(self, message: Union[TextMESSAGE, VoiceMESSAGE]):
         """
         Adds a message to the message list.
@@ -413,6 +332,7 @@ class GUILD(_BaseGUILD):
         await message.initialize(parent=self)
         self.message_dict[AdvertiseTaskType.TEXT_ISH if isinstance(message, TextMESSAGE) else AdvertiseTaskType.VOICE].append(message)
 
+    @typechecked
     def remove_message(self, message: Union[TextMESSAGE, VoiceMESSAGE]):
         """
         Removes a message from the message list.
@@ -440,14 +360,14 @@ class GUILD(_BaseGUILD):
 
         Raises
         -----------
-        DAFNotFoundError(code=DAF_SNOWFLAKE_NOT_FOUND)
+        ValueError
             Raised when the guild_id wasn't found.
 
         Other
             Raised from .add_message(message_object) method.
         """
         return await super().initialize(client.get_client().get_guild)
-
+    
     @misc._async_safe("update_semaphore", 2) # Take 2 since 2 tasks share access
     async def update(self, **kwargs):
         """
@@ -478,11 +398,11 @@ class GUILD(_BaseGUILD):
         await misc._update(self, **kwargs)
         # Update messages
         for message in self.messages:
-            await message.update(_init_options={"guild": self.apiobject})
+            await message.update(_init_options={"parent": self})
 
 
-@typechecked
-@sql._register_type("GuildTYPE")
+@misc.doc_category("Guilds")
+@sql.register_type("GuildTYPE")
 class USER(_BaseGUILD):
     """
     The USER object represents a user to whom messages will be sent.
@@ -506,16 +426,11 @@ class USER(_BaseGUILD):
         * timedelta - the specified time difference
         * datetime - specific date & time
     """
-
-    __logname__ = "USER" # For sql._register_type
     __slots__ = (
         "update_semaphore",
     )
 
-    @property
-    def _log_file_name(self):
-        return "".join(char if char not in C_FILE_NAME_FORBIDDEN_CHAR else "#" for char in f"{self.apiobject.display_name}#{self.apiobject.discriminator}") + ".json"
-
+    @typechecked
     def __init__(self,
                  snowflake: Union[int, discord.User],
                  messages: Optional[List[DirectMESSAGE]]=[],
@@ -524,6 +439,7 @@ class USER(_BaseGUILD):
         super().__init__(snowflake, messages, logging, remove_after)
         misc._write_attr_once(self, "update_semaphore", asyncio.Semaphore(2)) # Only allows re-referencing this attribute once
 
+    @typechecked
     async def add_message(self, message: DirectMESSAGE):
         """
         Adds a message to the message list.
@@ -546,6 +462,7 @@ class USER(_BaseGUILD):
         await message.initialize(parent=self)
         self.message_dict[AdvertiseTaskType.TEXT_ISH].append(message)
 
+    @typechecked
     def remove_message(self, message: DirectMESSAGE):
         """
         .. versionadded:: v2.0
@@ -570,7 +487,7 @@ class USER(_BaseGUILD):
 
         Raises
         -----------
-        DAFNotFoundError(code=DAF_SNOWFLAKE_NOT_FOUND)
+        ValueError
             Raised when the DM could not be created.
         Other
             Raised from .add_message(message_object) method.
@@ -607,9 +524,227 @@ class USER(_BaseGUILD):
         await misc._update(self, **kwargs)
         # Update messages
         for message in self.messages:
-            await message.update(_init_options={"user" : self.apiobject})
+            await message.update(_init_options={"parent" : self})
 
+@misc.doc_category("Automatic generation")
+class AutoGUILD:
+    """
+    .. versionadded:: v2.3
 
-async def _initialize(logging_path: str):
-    "Initializes the guild module"
-    GLOBALS.server_log_path = logging_path
\ No newline at end of file
+    Used for creating instances that will return
+    guild objects.
+
+    .. CAUTION::
+        Any objects passed to AutoGUILD get **deep-copied** meaning, those same objects
+        **will not be initialized** and cannot be used to obtain/change information regarding AutoGUILD.
+
+        .. code-block::
+            :caption: Illegal use of AutoGUILD
+            :emphasize-lines: 6, 7
+
+            auto_ch = daf.AutoCHANNEL(...)
+            tm = daf.TextMESSAGE(..., channels=auto_ch)
+
+            await daf.add_object(AutoGUILD(..., messages=[tm]))
+
+            auto_ch.channels # Illegal (does not represent the same object as in AutoGUILD), results in exception
+            await tm.update(...) # Illegal (does not represent the same object as in AutoGUILD), results in exception
+
+        To actually modify message/channel objects inside AutoGUILD, you need to iterate thru each GUILD.
+
+        .. code-block::
+            :caption: Modifying AutoGUILD messages
+
+            aguild = daf.AutoGUILD(..., messages=[tm])
+            await daf.add_object(aguild)
+
+            for guild in aguild.guilds:
+                for message in guild.messages
+                    await message.update(...)
+
+    Parameters
+    --------------
+    include_pattern: str
+        Regex pattern to use when searching guild names that are to be included.
+    exclude_pattern: Optional[str] = None
+        Regex pattern to use when searching guild names that are **NOT** to be excluded.
+
+        .. note::
+            If both include_pattern and exclude_pattern yield a match, the guild will be
+            excluded from match.
+
+    remove_after: Optional[Union[timedelta, datetime]] = None
+        When to remove this object from the shilling list.
+    logging: Optional[bool] = False
+        Set to True if you want the guilds generated to log
+        sent messages.
+    interval: Optional[timedelta] = timedelta(minutes=10)
+        Interval at which to scan for new guilds
+    """
+    __slots__ = (
+        "include_pattern",
+        "exclude_pattern",
+        "remove_after",
+        "messages",
+        "logging",
+        "interval",
+        "cache",
+        "last_scan",
+        "_created_at",
+        "_deleted",
+        "_safe_sem"
+    )
+
+    @typechecked
+    def __init__(self,
+                 include_pattern: str,
+                 exclude_pattern: Optional[str] = None,
+                 remove_after: Optional[Union[timedelta, datetime]] = None,
+                 messages: Optional[List[BaseMESSAGE]] = [],
+                 logging: Optional[bool] = False,
+                 interval: Optional[timedelta] = timedelta(minutes=10)) -> None:
+        self.include_pattern = include_pattern
+        self.exclude_pattern = exclude_pattern
+        self.remove_after = remove_after 
+        self.messages = messages # Uninitialized template messages list that gets copied for each found guild.
+        self.logging = logging
+        self.interval = interval.total_seconds() # In seconds
+        self.cache: Dict[GUILD] = {}
+        self.last_scan = 0
+        self._deleted = False
+        self._created_at = datetime.now()
+        misc._write_attr_once(self, "_safe_sem", asyncio.Semaphore(2))
+
+    @property
+    def guilds(self) -> List[GUILD]:
+        "Returns cached found GUILD objects."
+        return list(self.cache.values())
+
+    @property
+    def created_at(self) -> datetime:
+        """
+        Returns the datetime of when the object has been created.
+        """
+        return self._created_at
+
+    def _delete(self):
+        """
+        Sets the internal _deleted flag to True
+        and cancels main task.
+        """
+        self._deleted = True
+
+    def _check_state(self) -> bool:
+        """
+        Checks if the object is ready to be deleted.
+
+        If the object has already been deleted, return False 
+        to prevent multiple tasks from trying to remove it multiple
+        times which would result in ValueError exceptions.
+
+        Returns
+        ----------
+        True
+            The object should be deleted.
+        False
+            The object is in proper state, do not delete.
+        """
+        if self._deleted:
+            return False
+
+        rm_after_type = type(self.remove_after)
+        return (rm_after_type is timedelta and datetime.now() - self._created_at > self.remove_after or # The difference from creation time is bigger than remove_after
+                rm_after_type is datetime and datetime.now() > self.remove_after) # The current time is larger than remove_after 
+
+    async def initialize(self):
+        "Initializes asynchronous elements."
+        # Nothing is needed to be done here since everything is obtained from API wrapper cache.
+        pass 
+    
+    async def add_message(self, message: BaseMESSAGE):
+        """
+        Adds a copy of the passed message to each
+        guild inside cache.
+
+        Parameters
+        -------------
+        message: message.BaseMESSAGE
+            Message to add.
+
+        Raises
+        ---------
+        Any
+            Any exception raised in :py:meth:`daf.guild.GUILD.add_message`.
+        """
+        self.messages.append(message)
+        for guild in self.cache.values():
+            await guild.add_message(deepcopy(message))
+
+    def remove_message(self, message: BaseMESSAGE):
+        """
+        Removes message from the messages list.
+
+        Parameters
+        ------------
+        message: BaseMESSAGE
+            The message to remove.
+
+        Raises
+        --------
+        ValueError
+            The message is not present in the list.        
+        """
+        self.messages.remove(message)
+        for guild in self.guilds:
+            guild.remove_message(message)
+
+    async def _process(self):
+        """
+        Coroutine that finds new guilds from the
+        API wrapper.
+        """
+        dcl = client.get_client()
+        stamp = datetime.now().timestamp()
+        if stamp - self.last_scan > self.interval:
+            self.last_scan = stamp
+            for dcgld in dcl.guilds:
+                if (dcgld not in self.cache and 
+                    re.search(self.include_pattern, dcgld.name) is not None and
+                    (self.exclude_pattern is None or re.search(self.exclude_pattern, dcgld.name) is None)
+                ):
+                    try:
+                        new_guild = GUILD(snowflake=dcgld,
+                                                messages=deepcopy(self.messages),
+                                                logging=self.logging)
+                        await new_guild.initialize()
+                        self.cache[dcgld] = new_guild
+                    except Exception as exc:
+                        trace(f"[AutoGUILD:] Unable to add new object.\nReason: {exc}",TraceLEVELS.WARNING)
+
+    @misc._async_safe("_safe_sem", 1)
+    async def _advertise(self, type_: AdvertiseTaskType):
+        """
+        Advertises thru all the GUILDs.
+
+        Parameters
+        ----------------
+        type_: guild.AdvertiseTaskType
+            Which task called this method.
+            This is just forwarded to GUILDs' _advertise method.
+        """
+        await self._process()
+        for g in self.guilds:
+            await g._advertise(type_)
+
+    @misc._async_safe("_safe_sem", 2)
+    async def update(self, **kwargs):
+        """
+        Updates the object with new initialization parameters.
+
+        .. WARNING::
+            After calling this method the entire object is reset (this includes it's GUILD objects in cache).
+        """
+        if "interval" not in kwargs:
+            kwargs["interval"] = timedelta(seconds=self.interval)
+
+        return await misc._update(self, **kwargs)
\ No newline at end of file
